


from ephyviewer.standalone import StandAloneViewer, RawNeoOpenDialog
import pyqtgraph as pg



def test_StandAloneViewer():
    app = pg.mkQApp()
    win = StandAloneViewer()
    win.show()
    app.exec_()


def test_RawNeoOpenDialog():
    app = pg.mkQApp()
    dia = RawNeoOpenDialog()
    dia.show()
    app.exec_()
    
    print(dia.final_params)



if __name__=='__main__':
    test_StandAloneViewer()
    #~ test_RawNeoOpenDialog()



# -*- coding: utf-8 -*-
#~ from __future__ import (unicode_literals, print_function, division, absolute_import)
import numpy as np

import matplotlib.cm
import matplotlib.colors


from .sourcebase import BaseDataSource
from .events import BaseEventAndEpoch



class InMemoryEpochSource(BaseEventAndEpoch):
    type = 'Epoch'
    
    def __init__(self, all_epochs=[]):
        BaseEventAndEpoch.__init__(self, all=all_epochs)
        
        s = [ np.max(e['time']+e['duration']) for e in self.all if len(e['time'])>0]
        self._t_stop = max(s) if len(s)>0 else 0


    def get_chunk(self, chan=0,  i_start=None, i_stop=None):
        ep_times = self.all[chan]['time'][i_start:i_stop]
        ep_durations = self.all[chan]['duration'][i_start:i_stop]
        ep_labels = self.all[chan]['label'][i_start:i_stop]
        return ep_times, ep_durations, ep_labels
    
    def get_chunk_by_time(self, chan=0,  t_start=None, t_stop=None):
        ep_times = self.all[chan]['time']
        ep_durations = self.all[chan]['duration']
        ep_labels = self.all[chan]['label']
        
        #~ keep1 = (ep_times>=t_start) & (ep_times<t_stop)#begin inside
        #~ keep2 = (ep_times+ep_durations>=t_start) & (ep_times+ep_duqrations<t_stop) #end inside
        #~ keep3 = (ep_times<=t_start) & (ep_times+ep_durations>t_stop) # overlap
        #~ keep = keep1|  keep2 | keep3
        
        #~ return ep_times[keep], ep_durations[keep], ep_labels[keep]
        
        i1 = np.searchsorted(ep_times, t_start, side='left')
        i2 = np.searchsorted(ep_times+ep_durations, t_stop, side='left')
        sl = slice(max(0, i1-1), i2+1)
        return ep_times[sl], ep_durations[sl], ep_labels[sl]




class WritableEpochSource(InMemoryEpochSource):
    """
    Identique to EpochSource but onlye one channel that can be persisently saved.
    
    epoch is dict
    { 'time':np.array, 'duration':np.array, 'label':np.array, 'name':' ''}
    """
    def __init__(self, epoch, possible_labels, color_labels=None):
        InMemoryEpochSource.__init__(self, all_epochs=[epoch])
        
        #~ self._t_stop = max([ np.max(e['time']+e['duration']) for e in self.all if len(e['time'])>0])
        
        self.times = self.all[0]['time']
        self.durations = self.all[0]['duration']
        
        assert np.all((self.times[:-1]+self.durations[:-1])<=self.times[1:])
        
        assert np.all(np.in1d(epoch['label'], possible_labels))
        self.possible_labels = possible_labels
        
        if color_labels is None:
            n = len(possible_labels)
            cmap = matplotlib.cm.get_cmap('Dark2' , n)
            color_labels = [ matplotlib.colors.ColorConverter().to_rgb(cmap(i)) for i in  range(n)]
            color_labels = (np.array(color_labels)*255).astype(int)
            color_labels = color_labels.tolist()
        self.color_labels = color_labels
        self.label_to_color = dict(zip(self.possible_labels, self.color_labels))
        
    def get_chunk(self, chan=0,  i_start=None, i_stop=None):
        assert chan==0
        return InMemoryEpochSource. get_chunk(self, chan=chan,  i_start=i_start, i_stop=i_stop)
    
    def get_chunk_by_time(self, chan=0,  t_start=None, t_stop=None):
        #~ print(chan)
        assert chan==0
        return InMemoryEpochSource.get_chunk_by_time(self, chan=chan,  t_start=t_start, t_stop=t_stop)
    
    def color_by_label(self, label):
        return self.label_to_color[label]


    def _clean_and_set(self,ep_times, ep_durations, ep_labels):
        keep = ep_durations>0.
        ep_times = ep_times[keep]
        ep_durations = ep_durations[keep]
        ep_labels = ep_labels[keep]
        
        self.all[0]['time'] = ep_times
        self.all[0]['duration'] = ep_durations
        self.all[0]['label'] = ep_labels

    
    def add_epoch(self, t1, duration, label):
        ep_times, ep_durations, ep_labels = self.all[0]['time'], self.all[0]['duration'], self.all[0]['label']
        
        t2 = t1+duration

        ind = np.searchsorted(ep_times, t1, side='left')


        ep_times = insert_item(ep_times, ind, t1)
        ep_durations = insert_item(ep_durations, ind, duration)
        ep_labels = insert_item(ep_labels, ind, label)
        
        #previous
        prev = ind-1
        
        #if the previsous ends after the new ones then add the other part
        if ind>0:
            t2_prev = ep_times[prev]+ep_durations[prev]
            if (t2_prev)>t2:
                ep_times = insert_item(ep_times, ind+1, t2)
                ep_durations = insert_item(ep_durations, ind+1, t2_prev - t2)
                ep_labels = insert_item(ep_labels, ind+1, ep_labels[prev])
        #short prev durations
        while prev>=0:
            if (ep_times[prev]+ep_durations[prev])>ep_times[ind]:
                ep_durations[prev] = ep_times[ind] - ep_times[prev]
            else:
                break
            prev = prev-1
        
        #nexts
        next = ind+1
        while next<ep_times.size:
            delta = (ep_times[ind]+ep_durations[ind]) - ep_times[next]
            if delta>0:
                ep_times[next] += delta
                ep_durations[next] -= delta
            else:
                break
            next = next + 1
        
        self._clean_and_set(ep_times, ep_durations, ep_labels)
    
    def delete_in_between(self, t1, t2):
        #~ print('delete_in_between', t1, t2)
        ep_times, ep_durations, ep_labels = self.all[0]['time'], self.all[0]['duration'], self.all[0]['label']
        
        ep_stops = ep_times+ep_durations
        
        i1 = np.searchsorted(ep_times, t1, side='left')
        i2 = np.searchsorted(ep_times+ep_durations, t2, side='right')
        #~ print 'i1, i2', i1, i2
        #~ print()
        for i in range(i1-1, i2+1):
            #~ print i
            if i1<0: continue
            if i>=ep_durations.size: continue
            
            if ep_times[i]>=t1 and ep_stops[i]<t2:
                #~ print 'a'
                ep_durations[i] = -1.
            elif ep_times[i]<t1 and (t1<ep_stops[i]<t2):
                #~ print 'b'
                ep_durations[i] = t1 - ep_times[i]
            elif (t1<=ep_times[i]<t2) and ep_stops[i]>t2:
                #~ print 'c'
                ep_durations[i] = ep_stops[i] - t2
                ep_times[i] = t2
            elif ep_times[i]<=t1 and ep_stops[i]>=t2:
                #~ print 'd'
                ep_durations[i] = t1 - ep_times[i]
                # and insert one
                ep_times = insert_item(ep_times, i, t2)
                ep_durations = insert_item(ep_durations, i, ep_stops[i]-t2)
                ep_labels = insert_item(ep_labels,i, ep_labels[i])
                break
                
        
        self._clean_and_set(ep_times, ep_durations, ep_labels)
        
        
        

    def merge_neighbors(self):
        ep_times, ep_durations, ep_labels = self.all[0]['time'], self.all[0]['duration'], self.all[0]['label']
        
        mask = ((ep_times[:-1] + ep_durations[:-1])==ep_times[1:]) & (ep_labels[:-1]==ep_labels[1:])
        inds, = np.nonzero(mask)
        
        for ind in inds:
            ep_times[ind+1] = ep_times[ind]
            ep_durations[ind+1] = ep_durations[ind] + ep_durations[ind+1]
            ep_durations[ind] = -1
        
        self._clean_and_set(ep_times, ep_durations, ep_labels)
    
    def fill_blank(self, method='from_left'):
        ep_times, ep_durations, ep_labels = self.all[0]['time'], self.all[0]['duration'], self.all[0]['label']
        
        mask = ((ep_times[:-1] + ep_durations[:-1])<ep_times[1:])
        inds,  = np.nonzero(mask)
        
        if method=='from_left':
            for ind in inds:
                ep_durations[ind] = ep_times[ind+1] - ep_times[ind]
                
        elif method=='from_right':
            for ind in inds:
                gap = ep_times[ind+1] - (ep_times[ind] + ep_durations[ind])
                ep_times[ind+1] -= gap
                ep_durations[ind+1] += gap
                
        elif method=='from_nearest':
            for ind in inds:
                gap = ep_times[ind+1] - (ep_times[ind] + ep_durations[ind])
                ep_durations[ind] += gap/2.
                ep_times[ind+1] -= gap/2.
                ep_durations[ind+1] += gap/2.
                
        
        self._clean_and_set(ep_times, ep_durations, ep_labels)
    
    def save(self):
        print('WritableEpochSource.save')
    


        
        


def insert_item(arr, ind, value):
    new_arr = np.zeros(arr.size+1, dtype=arr.dtype)
    
    new_arr[:ind] = arr[:ind]
    new_arr[ind] = value
    new_arr[ind+1:] = arr[ind:]
    
    return new_arr



# -*- coding: utf-8 -*-
#~ from __future__ import (unicode_literals, print_function, division, absolute_import)

from collections import OrderedDict
import time
import sys
import pickle

from .myqt import QT, QT_MODE
from .navigation import NavigationToolBar

location_to_qt={
    'left': QT.LeftDockWidgetArea,
    'right': QT.RightDockWidgetArea,
    'top': QT.TopDockWidgetArea,
    'bottom': QT.BottomDockWidgetArea,
}

orientation_to_qt={
    'horizontal': QT.Horizontal,
    'vertical': QT.Vertical,
}


    

class MainViewer(QT.QMainWindow):
    def __init__(self, debug=False, settings_name=None, parent=None, **navigation_params):
        QT.QMainWindow.__init__(self, parent)

        #TODO settings
        #http://www.programcreek.com/python/example/86789/PyQt5.QtCore.QSettings
        
        self.debug = debug
        print('debug', debug)
        self.settings_name = settings_name
        if self.settings_name is not None:
            pyver = '.'.join(str(e) for e in sys.version_info[0:3])
            appname = 'ephyviewer'+'_py'+pyver
            self.settings = QT.QSettings(appname, self.settings_name)
        
        self.setDockNestingEnabled(True) 
        
        
        self.viewers = OrderedDict()
        
        self.navigation_toolbar = NavigationToolBar(**navigation_params)
        
        dock = self.navigation_dock =  QT.QDockWidget('navigation',self)
        dock.setObjectName( 'navigation')
        dock.setWidget(self.navigation_toolbar)
        dock.setTitleBarWidget(QT.QWidget())
        self.addDockWidget(QT.TopDockWidgetArea, dock)
        
        self.navigation_toolbar.time_changed.connect(self.on_time_changed)
        self.navigation_toolbar.xsize_changed.connect(self.on_xsize_changed)
        self.navigation_toolbar.auto_scale_requested.connect(self.auto_scale)
        
        self.load_one_setting('navigation_toolbar', self.navigation_toolbar)
        

    def add_view(self, widget, location='bottom', orientation='vertical',
                        tabify_with=None, split_with=None):
        name = widget.name
        
        assert name not in self.viewers, 'Viewer already in MainViewer'
        
        dock = QT.QDockWidget(name)
        dock.setObjectName(name)
        dock.setWidget(widget)
        
        #TODO chustum titlebar
        #~ dock.setTitleBarWidget(titlebar)
        
        if tabify_with is not None:
            assert tabify_with in self.viewers, '{} no exists'.format(tabify_with)
            #~ raise(NotImplementedError)
            #tabifyDockWidget ( QDockWidget * first, QDockWidget * second )
            other_dock = self.viewers[tabify_with]['dock']
            self.tabifyDockWidget(other_dock, dock)
            
        elif split_with is not None:
            assert split_with in self.viewers, '{} no exists'.format(split_with)
            #~ raise(NotImplementedError)
            orien = orientation_to_qt[orientation]
            other_dock = self.viewers[split_with]['dock']
            self.splitDockWidget(other_dock, dock, orien)
            #splitDockWidget ( QDockWidget * first, QDockWidget * second, Qt::Orientation orientation )
        else:
            loc = location_to_qt[location]
            orien = orientation_to_qt[orientation]
            #~ print(location, loc)
            self.addDockWidget(loc, dock, orien)

        self.viewers[name] = {'widget': widget, 'dock':dock}
        
        self.load_one_setting(name, widget)
        
        widget.time_changed.connect(self.on_time_changed)
        
        try:
            t_start = min(self.navigation_toolbar.t_start, widget.source.t_start)
            t_stop = max(self.navigation_toolbar.t_stop, widget.source.t_stop)
            #~ print('t_start, t_stop', t_start, t_stop, name)
            self.navigation_toolbar.set_start_stop(t_start, t_stop, seek=True)
        except:
            print('impossiblie to set t_start t_stop')
        
        
    
    def load_one_setting(self, name, widget):
        #~ print('load_one_setting', name, self.settings_name)
        if self.settings_name is not None:
            value = self.settings.value('viewer_'+name)
            #~ print('value', value)
            if value is not None:
                try:
                #~ if True:
                    if QT_MODE == 'PyQt4' and sys.version_info[0]==2:
                        if type(value)==QT.QVariant:
                            value = bytes(value.toPyObject())
                    value = pickle.loads(value)
                    widget.set_settings(value)
                except:
                    print('erreur load settings', name)
        
    
    def save_all_settings(self):
        print('save_all_settings')
        if self.settings_name is not None:
            for name, d in self.viewers.items():
                value = d['widget'].get_settings()
                #~ print('save', name, type(value))
                if value is not None:
                    #~ print('save ', name)
                    self.settings.setValue('viewer_'+name, pickle.dumps(value))
            value = self.navigation_toolbar.get_settings()
            if value is not None:
                #~ print('save ', 'navigation_toolbar')
                self.settings.setValue('viewer_navigation_toolbar', pickle.dumps(value))

    def on_time_changed(self, t):
        
        for name , viewer in self.viewers.items():
            if viewer['widget'] != self.sender():
                t0 = time.time()
                viewer['widget'].seek(t)
                
                if self.debug:
                    t1 = time.time()
                    print('refresh duration for', name, t1-t0, 's')
        
        if self.navigation_toolbar != self.sender():
            #~ print('self.navigation_toolbar.seek', t)
            self.navigation_toolbar.seek(t, emit=False)
    
    def on_xsize_changed(self, xsize):
        #~ print('on_xsize_changed', xsize)
        for name , viewer in self.viewers.items():
            if hasattr(viewer['widget'], 'set_xsize'):
                viewer['widget'].set_xsize(xsize)
    
    def auto_scale(self):
        #~ print('on_xsize_changed', xsize)
        for name , viewer in self.viewers.items():
            if hasattr(viewer['widget'], 'auto_scale'):
                viewer['widget'].auto_scale()
    
    
    def seek(self, t):
        for name , viewer in self.viewers.items():
            viewer['widget'].seek(t)
        
        self.navigation_toolbar.seek(t, emit=False)
    
    def set_xsize(self, xsize):
        if hasattr(self.navigation_toolbar, 'spinbox_xsize'):
            self.navigation_toolbar.spinbox_xsize.setValue(xsize)
        #~ self.on_xsize_changed(xsize)
    
    def closeEvent(self, event):
        for name , viewer in self.viewers.items():
            viewer['widget'].close()
        self.save_all_settings()
        event.accept()
        
        





import sys
import os
import argparse
from ephyviewer.datasource import HAVE_NEO
from ephyviewer.standalone import all_neo_rawio_dict, rawio_gui_params
    
def launch_standalone_ephyviewer():
    from ephyviewer.standalone import StandAloneViewer
    import pyqtgraph as pg
    assert HAVE_NEO, 'Must have neo 0.6.0'
    import neo
    
    
    argv = sys.argv[1:]
    
    parser = argparse.ArgumentParser(description='tridesclous')
    parser.add_argument('file_or_dir', help='', default=None, nargs='?')
    parser.add_argument('-f', '--format', help='File format', default=None)
    
    app = pg.mkQApp()
    win = StandAloneViewer()
    win.show()
    
    if len(argv)>=1:
        args = parser.parse_args(argv)
        file_or_dir_name = args.file_or_dir
        
        if args.format is None:
            #autoguess from extension
            neo_rawio_class = neo.rawio.get_rawio_class(file_or_dir_name)
        else:
            neo_rawio_class = all_neo_rawio_dict.get(args.format, None)
            
        assert neo_rawio_class is not None, 'Unknown format. Format list: ['+','.join(all_neo_rawio_dict.keys())+']'
        
        name = neo_rawio_class.__name__.replace('RawIO', '')
        if name in rawio_gui_params:
            raise(Exception('This IO needs arguments. Do ephyviewer and use open menu'))
        
        win.load_dataset(neo_rawio_class=neo_rawio_class, file_or_dir_names=[file_or_dir_name])
        
        
    app.exec_()

if __name__=='__main__':
    launch_standalone_ephyviewer()




# -*- coding: utf-8 -*-
#~ from __future__ import (unicode_literals, print_function, division, absolute_import)

from collections import OrderedDict
import numpy as np

import matplotlib.cm
import matplotlib.colors

from .myqt import QT
from . import tools

import pyqtgraph as pg

from .base import ViewerBase, Base_ParamController, MyViewBox, same_param_tree
from .epochviewer import RectItem, DataGrabber


default_params = [
    {'name': 'xsize', 'type': 'float', 'value': 3., 'step': 0.1},
    {'name': 'background_color', 'type': 'color', 'value': 'k'},
    {'name': 'new_epoch_step', 'type': 'float', 'value': .1, 'step': 0.1, 'limits':(0,np.inf)},
    {'name': 'view_mode', 'type': 'list', 'value':'stacked', 'values' : ['stacked', 'flat']},
    
    #~ {'name': 'display_labels', 'type': 'bool', 'value': True},
    ]



class EpochEncoder_ParamController(Base_ParamController):
    def __init__(self, parent=None, viewer=None, with_visible=True, with_color=True):
        Base_ParamController.__init__(self, parent=parent, viewer=viewer)
        
        self.resize(400, 600)

        h = QT.QHBoxLayout()
        self.mainlayout.addLayout(h)
        
        self.v1 = QT.QVBoxLayout()
        h.addLayout(self.v1)
        self.tree_label_params = pg.parametertree.ParameterTree()
        self.tree_label_params.setParameters(self.viewer.by_label_params, showTop=True)
        self.tree_label_params.header().hide()
        self.v1.addWidget(self.tree_label_params)

        #~ self.v1 = QT.QVBoxLayout()
        #~ h.addLayout(self.v1)
        #~ self.tree_params = pg.parametertree.ParameterTree()
        #~ self.tree_params.setParameters(self.viewer.params, showTop=True)
        #~ self.tree_params.header().hide()
        #~ self.v1.addWidget(self.tree_params)



class EpochEncoder(ViewerBase):
    _default_params = default_params
    
    request_data = QT.pyqtSignal(float, float, object)
    
    def __init__(self, **kargs):
        ViewerBase.__init__(self, **kargs)
        
        self.make_params()
        self.set_layout()
        self.make_param_controller()
        
        self.viewBox.doubleclicked.connect(self.show_params_controller)
        
        self._xratio = 0.3
        
        self.initialize_plot()
        
        self.on_range_visibility_changed(None, refresh=False)
        
        
        
        self.thread = QT.QThread(parent=self)
        self.datagrabber = DataGrabber(source=self.source)
        self.datagrabber.moveToThread(self.thread)
        self.thread.start()
        
        
        self.datagrabber.data_ready.connect(self.on_data_ready)
        self.request_data.connect(self.datagrabber.on_request_data)
        
        self.refresh_table()
        

    def make_params(self):
        # Create parameters
        self.params = pg.parametertree.Parameter.create(name='Global options',
                                                    type='group', children=self._default_params)
        self.params.param('xsize').setLimits((0, np.inf))
        
        
        keys = 'azertyuiop'
        all = []
        self.shortcuts = OrderedDict()
        for i, label in enumerate(self.source.possible_labels):
            key = keys[i] if i<len(keys) else ''
            
            name = 'label{}'.format(i)
            children =[{'name': 'name', 'type': 'str', 'value': self.source.possible_labels[i], 'readonly':True},
                            {'name': 'color', 'type': 'color', 'value': self.source.color_by_label(label)},
                            {'name': 'key', 'type': 'str', 'value': key},
                            ]
            all.append({'name': name, 'type': 'group', 'children': children})
            shortcut = QT.QShortcut (self)
            if key != '':
                shortcut.setKey(key)
            shortcut.activated.connect(self.on_shortcut)
            self.shortcuts[shortcut] = label
            
        self.by_label_params = pg.parametertree.Parameter.create(name='Labels', type='group', children=all)
        
        
        self.all_params = pg.parametertree.Parameter.create(name='all param',
                                    type='group', children=[self.params, self.by_label_params])
        
        self.params.sigTreeStateChanged.connect(self.on_param_change)
        self.by_label_params.sigTreeStateChanged.connect(self.on_change_keys)
    
    
    def set_layout(self):
        # layout
        self.mainlayout = QT.QVBoxLayout()
        self.setLayout(self.mainlayout)
        
        self.viewBox = MyViewBox()
        
        self.graphicsview  = pg.GraphicsView()#useOpenGL = True)
        self.mainlayout.addWidget(self.graphicsview, 4)
        
        self.plot = pg.PlotItem(viewBox=self.viewBox)
        self.plot.hideButtons()
        self.graphicsview.setCentralItem(self.plot)
        
        self.mainlayout.addSpacing(10)
        
        
        h = QT.QHBoxLayout()
        self.mainlayout.addLayout(h)
        
        g = QT.QGridLayout()
        h.addLayout(g)

        but = QT.PushButton('Colors and keys')
        g.addWidget(but, 0, 0)
        
        but.clicked.connect(self.show_params_controller)
        
        but = QT.PushButton('Merge neighbors')
        g.addWidget(but, 1, 0)
        but.clicked.connect(self.on_merge_neighbors)

        but = QT.PushButton('Fill blank')
        g.addWidget(but, 2, 0)
        but.clicked.connect(self.on_fill_blank)

        but = QT.PushButton('Save')
        g.addWidget(but, 4, 0)
        but.clicked.connect(self.on_save)
        
        
        #~ v.addStretch()
        #~ v.addWidget(QT.QFrame(frameShape=QT.QFrame.HLine, frameShadow=QT.QFrame.Sunken))
        
        self.but_range = QT.PushButton('Show/hide range', checkable=True)
        g.addWidget(self.but_range, 0, 1)
        self.but_range.clicked.connect(self.on_range_visibility_changed)
        
        spinboxs = []
        buts = []
        for i in range(2):
            hh = QT.QHBoxLayout()
            g.addLayout(hh, 1+i, 1)
            but = QT.QPushButton('>')
            buts.append(but)
            hh.addWidget(but)
            spinbox = pg.SpinBox(value=float(i), decimals = 8, bounds = (-np.inf, np.inf),step = 0.05, siPrefix=False, int=False)
            hh.addWidget(spinbox, 10)
            #~ g.addWidget(spinbox, 1+i, 1)
            spinbox.setSizePolicy(QT.QSizePolicy.Preferred, QT.QSizePolicy.Preferred, )
            spinbox.valueChanged.connect(self.on_spin_limit_changed)
            spinboxs.append(spinbox)
        self.spin_limit1, self.spin_limit2 = spinboxs
        buts[0].clicked.connect(self.set_limit1)
        buts[1].clicked.connect(self.set_limit2)
        
        
        self.combo_labels = QT.QComboBox()
        self.combo_labels.addItems(self.source.possible_labels)
        g.addWidget(self.combo_labels, 3, 1)
        
        self.but_apply_region = QT.PushButton('Apply')
        g.addWidget(self.but_apply_region, 4, 1)
        self.but_apply_region.clicked.connect(self.apply_region)

        self.but_del_region = QT.PushButton('Delete')
        g.addWidget(self.but_del_region, 5, 1)
        self.but_del_region.clicked.connect(self.delete_region)
        
        
        self.tree_params = pg.parametertree.ParameterTree()
        self.tree_params.setParameters(self.params, showTop=True)
        self.tree_params.header().hide()
        h.addWidget(self.tree_params)
        
        self.table_widget = QT.QTableWidget()
        h.addWidget(self.table_widget)
        self.table_widget.itemSelectionChanged.connect(self.on_seek_table)
        self.table_widget.setSelectionMode(QT.QAbstractItemView.SingleSelection)
        self.table_widget.setSelectionBehavior(QT.QAbstractItemView.SelectRows)
        

 
    def make_param_controller(self):
        self.params_controller = EpochEncoder_ParamController(parent=self, viewer=self)
        self.params_controller.setWindowFlags(QT.Qt.Window)


    def closeEvent(self, event):
        
        text = 'save ?'
        title = 'quit'
        mb = QT.QMessageBox.question(self, title,text, 
                QT.QMessageBox.Ok ,  QT.QMessageBox.Discard)
        if mb==QT.QMessageBox.Ok:
            self.source.save()
        
        self.thread.quit()
        self.thread.wait()
        event.accept()

    
    def initialize_plot(self):
        self.region = pg.LinearRegionItem(brush='#FF00FF20')
        self.region.setZValue(10)
        self.region.setRegion((0, 1.))
        self.plot.addItem(self.region, ignoreBounds=True)
        self.region.sigRegionChanged.connect(self.on_region_changed)

        self.vline = pg.InfiniteLine(angle=90, movable=False, pen='#00FF00')
        self.plot.addItem(self.vline)
        
        self.rect_items = []
        
        self.label_items = []
        for i, label in enumerate(self.source.possible_labels):
            color = self.by_label_params['label'+str(i), 'color']
            label_item = pg.TextItem(label, color=color, anchor=(0, 0.5), border=None, fill=pg.mkColor((128,128,128, 120)))
            self.plot.addItem(label_item)
            self.label_items.append(label_item)

        

    def show_params_controller(self):
        self.params_controller.show()
    
    def on_param_change(self):
        self.refresh()
    
    def on_change_keys(self, refresh=True):
        
        for i, label in enumerate(self.source.possible_labels):
            key = self.by_label_params['label'+str(i), 'key']
            shortcut = list(self.shortcuts.keys())[i]
            #~ print(shortcut)
            shortcut.setKey(key)
        
        self.refresh()
    
    def set_xsize(self, xsize):
        self.params['xsize'] = xsize


    def set_settings(self, value):
        #~ print('set_settings')
        actual_value = self.all_params.saveState()
        #~ print('same tree', same_param_tree(actual_value, value))
        if same_param_tree(actual_value, value):
            # this prevent restore something that is not same tree
            # as actual. Possible when new features.
            self.params.blockSignals(True)
            self.by_label_params.blockSignals(True)
            
            self.all_params.restoreState(value)
            
            self.params.blockSignals(False)
            self.by_label_params.blockSignals(False)
            self.on_change_keys(refresh=False)
        else:
            print('Not possible to restore setiings')
    
    def get_settings(self):
        return self.all_params.saveState()

    def refresh(self):
        xsize = self.params['xsize']
        t_start, t_stop = self.t-xsize*self._xratio , self.t+xsize*(1-self._xratio)
        self.request_data.emit(t_start, t_stop, [0])

    def on_data_ready(self, t_start, t_stop, visibles, data):
        #~ print('on_data_ready', self, t_start, t_stop, visibles, data)
        #~ self.plot.clear()
        
        for rect_item in self.rect_items:
            self.plot.removeItem(rect_item)
        self.rect_items = []
        
        self.graphicsview.setBackground(self.params['background_color'])
        
        times, durations, labels = data[0]
        #~ print(data)
        n = len(self.source.possible_labels)
        
        for i, label in enumerate(labels):
            ind = self.source.possible_labels.index(label)
            color = self.by_label_params['label'+str(ind), 'color']
            ypos = n - ind - 1
            if self.params['view_mode']=='stacked':
                ypos = n - ind - 1
            else:
                ypos = 0
            item = RectItem([times[i],  ypos,durations[i], .9],  border=color, fill=color)
            item = RectItem([times[i],  ypos,durations[i], .9],  border='#FFFFFF', fill=color)
            item.setPos(times[i],  ypos)
            self.plot.addItem(item)
            self.rect_items.append(item)
        
        
        #~ for i, label in enumerate(self.source.possible_labels):
        for i, label_item in enumerate(self.label_items):
            if self.params['view_mode']=='stacked':
                color = self.by_label_params['label'+str(i), 'color']
                #~ label_item = pg.TextItem(label, color=color, anchor=(0, 0.5), border=None, fill=pg.mkColor((128,128,128, 120)))
                label_item.setColor(color)
                label_item.setPos(t_start, n - i - 0.55)
                label_item.show()
                #~ self.plot.addItem(label_item)
            else:
                label_item.hide()
        
        if self.but_range.isChecked():
            self.region.show()
        else:
            self.region.hide()
        
        self.vline.setPos(self.t)
        self.plot.setXRange( t_start, t_stop)
        if self.params['view_mode']=='stacked':
            self.plot.setYRange( 0, n)
        else:
            self.plot.setYRange( 0, 1)
    
    def on_shortcut(self):
        label = self.shortcuts.get(self.sender(), None)
        if label is None: return
        
        #~ duration = self.spin_step.value()
        duration = self.params['new_epoch_step']
        
        self.source.add_epoch(self.t, duration, label)
        
        self.t += duration
        self.refresh()
        self.time_changed.emit(self.t)
        self.refresh_table()
    
    def on_merge_neighbors(self):
        self.source.merge_neighbors()
        self.refresh()
    
    def on_fill_blank(self):
        params = [{'name': 'method', 'type': 'list', 'value':'from_left', 'values' : ['from_left', 'from_right', 'from_nearest']}]
        dia = tools.ParamDialog(params, title='Fill blank method', parent=self)
        dia.resize(300, 100)
        if dia.exec_():
            d = dia.get()
            method = d['method']
            self.source.fill_blank(method=method)
            self.refresh()
            self.refresh_table()
    
    
    def on_save(self):
        self.source.save()
    
    def on_spin_limit_changed(self, v):
        self.region.blockSignals(True)
        rgn = (self.spin_limit1.value(), self.spin_limit2.value())
        rgn = self.region.setRegion(rgn)
        self.region.blockSignals(False)
    
    def on_region_changed(self):
        self.spin_limit1.blockSignals(True)
        self.spin_limit2.blockSignals(True)
        rgn = self.region.getRegion()
        self.spin_limit1.setValue(rgn[0])
        self.spin_limit2.setValue(rgn[1])
        self.spin_limit1.blockSignals(False)
        self.spin_limit2.blockSignals(False)

    def apply_region(self):
        rgn = self.region.getRegion()
        t = rgn[0]
        duration = rgn[1] - rgn[0]
        label = str(self.combo_labels.currentText())
        self.source.add_epoch(t, duration, label)
        
        self.refresh()
        self.refresh_table()
    
    def delete_region(self):
        rgn = self.region.getRegion()
        
        self.source.delete_in_between(rgn[0], rgn[1])
        
        self.refresh()
        self.refresh_table()
        
    
    def on_range_visibility_changed(self, flag, refresh=True):
        enabled = self.but_range.isChecked()
        #~ print(enabled)
        for w in (self.spin_limit1, self.spin_limit2, self.combo_labels, self.but_apply_region, self.but_del_region):
            w.setEnabled(enabled)
            
        if enabled:
            rgn = self.region.getRegion()
            rgn = (self.t, self.t + rgn[1] - rgn[0])
            self.region.setRegion(rgn)
        self.refresh()

    def set_limit1(self):
        if self.t<self.spin_limit2.value():
            self.spin_limit1.setValue(self.t)

    def set_limit2(self):
        if self.t>self.spin_limit1.value():
            self.spin_limit2.setValue(self.t)
    
    def refresh_table(self):
        self.table_widget.clear()
        #~ ev = self.source.all_events[ind]
        times, durations, labels = self.source.get_chunk(chan=0,  i_start=None, i_stop=None)
        self.table_widget.setColumnCount(4)
        self.table_widget.setRowCount(times.size)
        self.table_widget.setHorizontalHeaderLabels(['start', 'stop', 'duration', 'label'])
        for r in range(times.size):
            
            values = [times[r], times[r]+durations[r], durations[r], labels[r]]
            for c, value in enumerate(values):
                item = QT.QTableWidgetItem('{}'.format(value))
                item.setFlags(QT.ItemIsSelectable|QT.ItemIsEnabled)
                self.table_widget.setItem(r, c, item)
    
    def on_seek_table(self):
        if self.table_widget.rowCount()==0:
            return
        selected_ind = self.table_widget.selectedIndexes()
        if len(selected_ind)==0:
            return
        i = selected_ind[0].row()
        t, _, _= self.source.get_chunk(chan=0,  i_start=i, i_stop=i+1)
        self.t = t[0]
        self.refresh()
        self.time_changed.emit(self.t)
        
        





# -*- coding: utf-8 -*-
"""


"""

from .sourcebase import BaseDataSource
import sys
import logging

import numpy as np

try:
    from distutils.version import LooseVersion as V
    import neo
    if V(neo.__version__)>='0.6.0':
        HAVE_NEO = True
        from neo.rawio.baserawio import BaseRawIO
    else:
        HAVE_NEO = False
        #~ print('neo version is too old', neo.__version__)
except ImportError:
    HAVE_NEO = False

from .signals import BaseAnalogSignalSource
from .events import BaseEventAndEpoch
from .spikes import BaseSpikeSource


logger = logging.getLogger()

#~ print('HAVE_NEO', HAVE_NEO)

class NeoAnalogSignalSource(BaseAnalogSignalSource):
    def __init__(self, neorawio, channel_indexes=None):
        
        BaseAnalogSignalSource.__init__(self)
        self.with_scatter = False
        
        self.neorawio =neorawio
        if channel_indexes is None:
            channel_indexes = slice(None)
        self.channel_indexes = channel_indexes
        self.channels = self.neorawio.header['signal_channels'][channel_indexes]
        self.sample_rate = self.neorawio.get_signal_sampling_rate(channel_indexes=self.channel_indexes)
        
        #TODO: something for multi segment
        self.block_index = 0
        self.seg_index = 0
    
    @property
    def nb_channel(self):
        return len(self.channels)

    def get_channel_name(self, chan=0):
        return self.channels[chan]['name']

    @property
    def t_start(self):
        t_start = self.neorawio.get_signal_t_start(self.block_index, self.seg_index,
                    channel_indexes=self.channel_indexes)
        return t_start

    @property
    def t_stop(self):
        t_stop = self.t_start + self.get_length()/self.sample_rate
        return t_stop
    
    def get_length(self):
        length = self.neorawio.get_signal_size(self.block_index, self.seg_index,
                        channel_indexes=self.channel_indexes)
        return length

    def get_shape(self):
        return (self.get_length(), self.nb_channel)

    def get_chunk(self, i_start=None, i_stop=None):
        sigs = self.neorawio.get_analogsignal_chunk(block_index=self.block_index, seg_index=self.seg_index,
                        i_start=i_start, i_stop=i_stop, channel_indexes=self.channel_indexes)
        #TODO something for scaling
        
        #TODO add an option to pre load evrything in memory for short length
        
        return sigs
        

class NeoSpikeSource(BaseSpikeSource):
    def __init__(self, neorawio, channel_indexes=None):
        self.neorawio =neorawio
        if channel_indexes is None:
            channel_indexes = slice(None)
        self.channel_indexes = channel_indexes
        
        self.channels = self.neorawio.header['unit_channels'][channel_indexes]
        
        #TODO: something for multi segment
        self.block_index = 0
        self.seg_index = 0

    @property
    def nb_channel(self):
        return len(self.channels)

    def get_channel_name(self, chan=0):
        return self.channels[chan]['name']

    @property
    def t_start(self):
        t_start = self.neorawio.segment_t_start(self.block_index, self.seg_index)
        return t_start

    @property
    def t_stop(self):
        t_stop = self.neorawio.segment_t_stop(self.block_index, self.seg_index)
        return t_stop

    def get_chunk(self, chan=0,  i_start=None, i_stop=None):
        raise(NotImplementedError)
    
    def get_chunk_by_time(self, chan=0,  t_start=None, t_stop=None):
        spike_timestamp = self.neorawio.get_spike_timestamps(block_index=self.block_index,
                        seg_index=self.seg_index, unit_index=chan, t_start=t_start, t_stop=t_stop)
        
        spike_times = self.neorawio.rescale_spike_timestamp(spike_timestamp, dtype='float64')
        
        return spike_times

    


class NeoEpochSource(BaseEventAndEpoch):
    def __init__(self, neorawio, channel_indexes=None):
        self.neorawio =neorawio
        if channel_indexes is None:
            channel_indexes = slice(None)
        self.channel_indexes = channel_indexes
        
        self.channels = self.neorawio.header['event_channels'][channel_indexes]
        
        #TODO: something for multi segment
        self.block_index = 0
        self.seg_index = 0
        
        self._cache_event = {}

    @property
    def nb_channel(self):
        return len(self.channels)

    def get_channel_name(self, chan=0):
        return self.channels[chan]['name']

    @property
    def t_start(self):
        t_start = self.neorawio.segment_t_start(self.block_index, self.seg_index)
        return t_start

    @property
    def t_stop(self):
        t_stop = self.neorawio.segment_t_stop(self.block_index, self.seg_index)
        return t_stop

    def get_chunk(self, chan=0,  i_start=None, i_stop=None):
        k = (self.block_index , self.seg_index, chan) 
        if k not in self._cache_event:
            self._cache_event[k] = self.get_chunk_by_time(chan=chan,  t_start=None, t_stop=None) 
        
        ep_times, ep_durations, ep_labels = self._cache_event[k]
        
        ep_times = ep_times[i_start:i_stop]
        if ep_durations is not None:
            ep_durations = ep_durations[i_start:i_stop]
        ep_labels = ep_labels[i_start:i_stop]
        
        return ep_times, ep_durations, ep_labels
    
    def get_chunk_by_time(self, chan=0,  t_start=None, t_stop=None):
        
        ep_timestamps, ep_durations, ep_labels = self.neorawio.get_event_timestamps(block_index=self.block_index,
                        seg_index=self.seg_index, event_channel_index=chan, t_start=t_start, t_stop=t_stop)
        
        ep_times = self.neorawio.rescale_event_timestamp(ep_timestamps, dtype='float64')
        
        if ep_durations is not None:
            ep_durations = self.neorawio.rescale_epoch_duration(ep_durations, dtype='float64')
        else:
            ep_durations = np.zeros_like(ep_times)
        
        return ep_times, ep_durations, ep_labels


    
    
    
def get_source_from_neo(neorawio):
    assert HAVE_NEO
    assert isinstance(neorawio, BaseRawIO)
    
    if neorawio.header is None:
        logger.info('parse header')
        neorawio.parse_header()
    
    sources = {'signal':[], 'epoch':[], 'spike':[]}
    
    if neorawio.signal_channels_count()>0:
        #Signals
        for channel_indexes in neorawio.get_group_channel_indexes():
            #one soure by channel group
            sources['signal'].append(NeoAnalogSignalSource(neorawio, channel_indexes))
            
        
    
    if neorawio.unit_channels_count()>0:
        #Spikes: TODO
        sources['spike'].append(NeoSpikeSource(neorawio, None))
    
    
    if neorawio.event_channels_count()>0:
        sources['epoch'].append(NeoEpochSource(neorawio, None))
        
    
    
    return sources
    
    


# -*- coding: utf-8 -*-
#~ from __future__ import (unicode_literals, print_function, division, absolute_import)


import numpy as np

import matplotlib.cm
import matplotlib.colors

from .myqt import QT
import pyqtgraph as pg

from .base import BaseMultiChannelViewer, Base_MultiChannel_ParamController
from .datasource import InMemoryEpochSource



#make symbol for spikes
from pyqtgraph.graphicsItems.ScatterPlotItem import Symbols
Symbols['|'] = QT.QPainterPath()
Symbols['|'].moveTo(0, -0.5)
Symbols['|'].lineTo(0, 0.5)
Symbols['|'].closeSubpath()




default_params = [
    {'name': 'xsize', 'type': 'float', 'value': 3., 'step': 0.1},
    {'name': 'background_color', 'type': 'color', 'value': 'k'},
    {'name': 'display_labels', 'type': 'bool', 'value': True},
    ]

default_by_channel_params = [ 
    {'name': 'color', 'type': 'color', 'value': "55FF00"},
    {'name': 'visible', 'type': 'bool', 'value': True},
    ]




class SpikeTrainViewer_ParamController(Base_MultiChannel_ParamController):
    pass



class DataGrabber(QT.QObject):
    data_ready = QT.pyqtSignal(float, float, object, object)
    
    def __init__(self, source, parent=None):
        QT.QObject.__init__(self, parent)
        self.source = source
        
    def on_request_data(self, t_start, t_stop, visibles):
        data = {}
        for e, chan in enumerate(visibles):
            times = self.source.get_chunk_by_time(chan=chan,  t_start=t_start, t_stop=t_stop)
            data[chan] = times
        self.data_ready.emit(t_start, t_stop, visibles, data)
    

class SpikeTrainViewer(BaseMultiChannelViewer):
    _default_params = default_params
    _default_by_channel_params = default_by_channel_params
    
    _ControllerClass = SpikeTrainViewer_ParamController
    
    request_data = QT.pyqtSignal(float, float, object)
    
    def __init__(self, **kargs):
        BaseMultiChannelViewer.__init__(self, **kargs)
        
        self.make_params()
        self.set_layout()
        self.make_param_controller()
        
        self.viewBox.doubleclicked.connect(self.show_params_controller)
        
        self._xratio = 0.3
        
        self.initialize_plot()
        
        
        
        self.thread = QT.QThread(parent=self)
        self.datagrabber = DataGrabber(source=self.source)
        self.datagrabber.moveToThread(self.thread)
        self.thread.start()
        
        
        self.datagrabber.data_ready.connect(self.on_data_ready)
        self.request_data.connect(self.datagrabber.on_request_data)
        
        self.params.param('xsize').setLimits((0, np.inf))

    @classmethod
    def from_numpy(cls, all_epochs, name):
        source = InMemoryEpochSource(all_epochs)
        view = cls(source=source, name=name)
        return view
    
    def closeEvent(self, event):
        event.accept()
        self.thread.quit()
        self.thread.wait()

    
    def initialize_plot(self):
        self.vline = pg.InfiniteLine(angle = 90, movable = False, pen = '#00FF00')
        self.plot.addItem(self.vline)
        
        self.scatter = pg.ScatterPlotItem(size=10, pxMode = True, symbol='|')
        self.plot.addItem(self.scatter)
        
        
        self.labels = []
        for c in range(self.source.nb_channel):
            label_name = '{}: {}'.format(c, self.source.get_channel_name(chan=c))
            color = self.by_channel_params.children()[c].param('color').value()
            label = pg.TextItem(label_name, color=color, anchor=(0, 0.5), border=None, fill=pg.mkColor((128,128,128, 180)))
            self.plot.addItem(label)
            self.labels.append(label)
    
    def refresh(self):
        xsize = self.params['xsize']
        t_start, t_stop = self.t-xsize*self._xratio , self.t+xsize*(1-self._xratio)
        visibles, = np.nonzero(self.params_controller.visible_channels)
        self.request_data.emit(t_start, t_stop, visibles)

    def on_data_ready(self, t_start, t_stop, visibles, data):
        self.graphicsview.setBackground(self.params['background_color'])
        
        self.scatter.clear()
        all_x = []
        all_y = []
        all_brush = []
        
        for e, chan in enumerate(visibles):
            times = data[chan]

            ypos = visibles.size-e-1
            
            all_x.append(times)
            all_y.append(np.ones(times.size)*ypos)
            color = self.by_channel_params.children()[chan].param('color').value()
            all_brush.append(np.array([pg.mkPen(color)]*len(times)))
            
            
            if self.params['display_labels']:
                self.labels[chan].setPos(t_start, ypos)
                self.labels[chan].show()
                self.labels[chan].setColor(color)

        for chan in range(self.source.nb_channel):
            if not self.params['display_labels'] or chan not in visibles:
                self.labels[chan].hide()

        if len(all_x):
            all_x = np.concatenate(all_x)
            all_y = np.concatenate(all_y)
            all_brush = np.concatenate(all_brush)
            self.scatter.setData(x=all_x, y=all_y, pen=all_brush)

        self.vline.setPos(self.t)
        
        self.plot.setXRange( t_start, t_stop)
        self.plot.setYRange( 0, visibles.size)





# -*- coding: utf-8 -*-
#~ from __future__ import (unicode_literals, print_function, division, absolute_import)


import numpy as np

from .myqt import QT
import pyqtgraph as pg

from .base import ViewerBase
from .datasource import InMemoryEventSource




class EventList(ViewerBase):
    
    def __init__(self,  **kargs):
        ViewerBase.__init__(self, **kargs)
        
        self.mainlayout = QT.QVBoxLayout()
        self.setLayout(self.mainlayout)
        
        
        self.combo = QT.QComboBox()
        self.mainlayout.addWidget(self.combo)
        self.list_widget = QT.QListWidget()
        self.mainlayout.addWidget(self.list_widget)
        self.combo.currentIndexChanged.connect(self.refresh_list)
        
        self.combo.addItems([self.source.get_channel_name(i) for i in range(self.source.nb_channel) ])
        
        self.list_widget.currentRowChanged.connect(self.select_event)

    @classmethod
    def from_numpy(cls, all_events, name):
        source = InMemoryEventSource(all_events)
        view = cls(source=source, name=name)
        return view
        
    def refresh(self):
        pass
        
    def refresh_list(self, ind):
        self.ind = ind
        self.list_widget.clear()
        #~ ev = self.source.all_events[ind]
        data = self.source.get_chunk(chan=ind,  i_start=None, i_stop=None)
        
        if len(data)==2:
            times, labels = data
        elif len(data)==3:
            times, _, labels = data
        
        for i in range(times.size):
            if labels is None:
                self.list_widget.addItem('{} : {:.3f}'.format(i, times[i]) )
            else:
                self.list_widget.addItem('{} : {:.3f} {}'.format(i, times[i], labels[i]) )

        
    def select_event(self, i):
        
        #~ ev = self.source.all_events[self.ind]
        #~ t = ev['time'][i]
        data = self.source.get_chunk(chan=self.ind,  i_start=i, i_stop=i+1)
        
        if len(data)==2:
            times, labels = data
        elif len(data)==3:
            times, _, labels = data
        
        if len(times)>0:
            t = float(times[0])
            self.time_changed.emit(t)
        


# -*- coding: utf-8 -*-
#~ from __future__ import (unicode_literals, print_function, division, absolute_import)


import numpy as np

#~ import matplotlib.cm
#~ import matplotlib.colors

from .myqt import QT
import pyqtgraph as pg

from .base import BaseMultiChannelViewer, Base_MultiChannel_ParamController
from .datasource import InMemoryAnalogSignalSource, AnalogSignalSourceWithScatter


#todo remove this
import time
import threading

MIN_XSIZE = 1e-6


default_params = [
    {'name': 'xsize', 'type': 'float', 'value': 3., 'step': 0.1},
    {'name': 'ylim_max', 'type': 'float', 'value': 10.},
    {'name': 'ylim_min', 'type': 'float', 'value': -10.},
    {'name': 'scale_mode', 'type': 'list', 'value': 'real_scale', 
        'values':['real_scale', 'same_for_all', 'by_channel'] },
    {'name': 'background_color', 'type': 'color', 'value': 'k'},
    {'name': 'display_labels', 'type': 'bool', 'value': False},
    {'name': 'display_offset', 'type': 'bool', 'value': False},
    {'name': 'antialias', 'type': 'bool', 'value': False},    
    {'name': 'decimation_method', 'type': 'list', 'value': 'min_max', 'values': ['min_max', 'mean', 'pure_decimate',  ]},
    ]

default_by_channel_params = [ 
    {'name': 'color', 'type': 'color', 'value': "55FF00"},
    {'name': 'gain', 'type': 'float', 'value': 1, 'step': 0.1},
    {'name': 'offset', 'type': 'float', 'value': 0., 'step': 0.1},
    {'name': 'visible', 'type': 'bool', 'value': True},
    ]




#TODO use Base_MultiChannel_ParamController instead of Base_ParamController
#to avoid code duplication



class TraceViewer_ParamController(Base_MultiChannel_ParamController):
    
    def __init__(self, parent=None, viewer=None):
        Base_MultiChannel_ParamController.__init__(self, parent=parent, viewer=viewer, with_visible=True, with_color=True)
        
        
        #TODO put this somewhere
        
        #~ v.addWidget(QT.QLabel(self.tr('<b>Gain zoom (mouse wheel on graph):</b>'),self))
        #~ h = QT.QHBoxLayout()
        #~ v.addLayout(h)
        #~ for label, factor in [('--', 1./5.), ('-', 1./1.1), ('+', 1.1), ('++', 5.),]:
            #~ but = QT.QPushButton(label)
            #~ but.factor = factor
            #~ but.clicked.connect(self.on_but_ygain_zoom)
            #~ h.addWidget(but)
        
        #~ self.ygain_factor = 1.
        
    @property
    def selected(self):
        selected = np.ones(self.viewer.source.nb_channel, dtype=bool)
        if self.viewer.source.nb_channel>1:
            selected[:] = False
            selected[[ind.row() for ind in self.qlist.selectedIndexes()]] = True
        return selected
    
    @property
    def visible_channels(self):
        visible = [self.viewer.by_channel_params['ch{}'.format(i), 'visible'] for i in range(self.source.nb_channel)]
        return np.array(visible, dtype='bool')

    @property
    def gains(self):
        gains = [self.viewer.by_channel_params['ch{}'.format(i), 'gain'] for i in range(self.source.nb_channel)]
        return np.array(gains)

    @gains.setter
    def gains(self, val):
        for c, v in enumerate(val):
            self.viewer.by_channel_params['ch{}'.format(c), 'gain'] = v

    @property
    def offsets(self):
        offsets = [self.viewer.by_channel_params['ch{}'.format(i), 'offset'] for i in range(self.source.nb_channel)]
        return np.array(offsets)

    @offsets.setter
    def offsets(self, val):
        for c, v in enumerate(val):
            self.viewer.by_channel_params['ch{}'.format(c), 'offset'] = v

        
    
    def estimate_median_mad(self):
        sigs = self.viewer.last_sigs_chunk
        assert sigs is not None, 'Need to debug this'
        self.signals_med = med = np.median(sigs, axis=0)
        self.signals_mad = np.median(np.abs(sigs-med),axis=0)*1.4826
        #~ print('estimate_median_mad')
        #~ print('self.signals_med', self.signals_med)

    def compute_rescale(self):
        scale_mode = self.viewer.params['scale_mode']
        #~ print('compute_rescale', scale_mode)
        
        self.viewer.all_params.blockSignals(True)
        
        gains = np.ones(self.viewer.source.nb_channel)
        offsets = np.zeros(self.viewer.source.nb_channel)
        nb_visible = np.sum(self.visible_channels)
        #~ self.ygain_factor = 1
        if scale_mode=='real_scale':
            self.viewer.params['ylim_min'] = np.min(self.viewer.last_sigs_chunk)
            self.viewer.params['ylim_max'] = np.max(self.viewer.last_sigs_chunk)
        else:
            if self.viewer.last_sigs_chunk is not None:
                self.estimate_median_mad()
                if scale_mode=='same_for_all':
                    gains[self.visible_channels] = np.ones(nb_visible, dtype=float) / max(self.signals_mad[self.visible_channels]) / 9.
                elif scale_mode=='by_channel':
                    gains[self.visible_channels] = np.ones(nb_visible, dtype=float) / self.signals_mad[self.visible_channels] / 9.
                offsets[self.visible_channels] = np.arange(nb_visible)[::-1] - self.signals_med[self.visible_channels]*gains[self.visible_channels]
                self.viewer.params['ylim_min'] = -0.5
                self.viewer.params['ylim_max'] = nb_visible-0.5
            
        self.gains = gains
        self.offsets = offsets
        self.viewer.all_params.blockSignals(False)        

    def on_channel_visibility_changed(self):
        #~ print('on_channel_visibility_changed')
        self.compute_rescale()
        self.viewer.refresh()
    
    def on_but_ygain_zoom(self):
        factor = self.sender().factor
        self.apply_ygain_zoom(factor)

    def apply_ygain_zoom(self, factor_ratio):
        
        scale_mode = self.viewer.params['scale_mode']
        
        self.viewer.all_params.blockSignals(True)
        if scale_mode=='real_scale':
            #~ self.ygain_factor *= factor_ratio
            
            self.viewer.params['ylim_max'] = self.viewer.params['ylim_max']*factor_ratio
            self.viewer.params['ylim_min'] = self.viewer.params['ylim_min']*factor_ratio
            
            pass
            #TODO ylims
        else :
            #~ self.ygain_factor *= factor_ratio
            if not hasattr(self, 'self.signals_med'):
                self.estimate_median_mad()
            self.gains = self.gains * factor_ratio
            self.offsets = self.offsets + self.signals_med*self.gains * (1-factor_ratio)
        
        self.viewer.all_params.blockSignals(False)
        
        self.viewer.refresh()
        #~ print('apply_ygain_zoom', factor_ratio)#, 'self.ygain_factor', self.ygain_factor)
        
    def apply_xsize_zoom(self, xmove):
        factor = xmove/100.
        factor = max(factor, -0.999999999)
        factor = min(factor, 1)
        newsize = self.viewer.params['xsize']*(factor+1.)
        self.viewer.params['xsize'] = max(newsize, MIN_XSIZE)



class DataGrabber(QT.QObject):
    data_ready = QT.pyqtSignal(float, float, float, object, object, object, object, object)
    
    def __init__(self, source, viewer, parent=None):
        QT.QObject.__init__(self, parent)
        self.source = source
        self.viewer = viewer
        self._max_point = 3000
    
    def get_data(self, t, t_start, t_stop, gains, offsets, visibles, decimation_method):

        i_start, i_stop = self.source.time_to_index(t_start), self.source.time_to_index(t_stop)
        #~ print(t_start, t_stop, i_start, i_stop)
        
        ds_ratio = (i_stop - i_start)//self._max_point + 1
        #~ print()
        #~ print('ds_ratio', ds_ratio, 'i_start i_stop', i_start, i_stop  )
        
        if ds_ratio>1:
            i_start = i_start - (i_start%ds_ratio)
            i_stop = i_stop - (i_stop%ds_ratio)
            #~ print('i_start, i_stop', i_start, i_stop)
        
        #clip it
        i_start = max(0, i_start)
        i_start = min(i_start, self.source.get_length())
        i_stop = max(0, i_stop)
        i_stop = min(i_stop, self.source.get_length())
        if ds_ratio>1:
            #after clip
            i_start = i_start - (i_start%ds_ratio)
            i_stop = i_stop - (i_stop%ds_ratio)
        
        #~ print('final i_start i_stop', i_start, i_stop  )
        
        sigs_chunk = self.source.get_chunk(i_start=i_start, i_stop=i_stop)
        
        
        
        #~ print('sigs_chunk.shape', sigs_chunk.shape)
        data_curves = sigs_chunk[:, visibles].T.copy()
        if data_curves.dtype!='float32':
            data_curves = data_curves.astype('float32')
        
        if ds_ratio>1:
            
            
            small_size = (data_curves.shape[1]//ds_ratio)
            if decimation_method == 'min_max':
                small_size *= 2
            
            small_arr = np.empty((data_curves.shape[0], small_size), dtype=data_curves.dtype)
            
            if decimation_method == 'min_max':
                full_arr = data_curves.reshape(data_curves.shape[0], -1, ds_ratio)
                small_arr[:, ::2] = full_arr.max(axis=2)
                small_arr[:, 1::2] = full_arr.min(axis=2)
            elif decimation_method == 'mean':
                full_arr = data_curves.reshape(data_curves.shape[0], -1, ds_ratio)
                small_arr[:, :] = full_arr.mean(axis=2)
            elif decimation_method == 'pure_decimate':
                small_arr[:, :] = data_curves[:, ::ds_ratio]
            data_curves = small_arr
        
        #~ print(data_curves.shape)
        
        data_curves *= gains[visibles, None]
        data_curves += offsets[visibles, None]
        dict_curves ={}
        for i, c in enumerate(visibles):
            dict_curves[c] = data_curves[i, :]
        
        #~ print(ds_ratio)
        t_start2 = self.source.index_to_time(i_start)
        times_curves = np.arange(data_curves.shape[1], dtype='float32')
        times_curves /= self.source.sample_rate/ds_ratio
        if ds_ratio>1 and decimation_method == 'min_max':
            times_curves /=2
        times_curves += t_start2
        
        dict_scatter = None
        if self.source.with_scatter:
            pass
            dict_scatter = {}
            for k in self.source.get_scatter_babels():
                x, y = [], []
                for i, c in enumerate(visibles):
                    scatter_inds = self.source.get_scatter(i_start=i_start, i_stop=i_stop, chan=c, label=k)
                    if scatter_inds is None: continue
                    x.append((scatter_inds-i_start)/self.source.sample_rate+t_start2)
                    y.append(sigs_chunk[scatter_inds-i_start, c]*gains[c]+offsets[c])
                    
                dict_scatter[k] = (np.concatenate(x), np.concatenate(y))
                    
        return t, t_start, t_stop, visibles, dict_curves, times_curves, sigs_chunk, dict_scatter
    
    def on_request_data(self, t, t_start, t_stop, gains, offsets, visibles, decimation_method):
        #~ print('on_request_data', t_start, t_stop)
        
        if self.viewer.t != t:
            #~ print('on_request_data not same t')
            return
        
        t, t_start, t_stop, visibles, dict_curves, times_curves,\
                    sigs_chunk, dict_scatter = self.get_data(t, t_start, t_stop, gains, offsets, visibles, decimation_method)
                    
        
        #~ print('on_request_data', threading.get_ident())
        #~ time.sleep(1.)
        self.data_ready.emit(t, t_start, t_stop, visibles, dict_curves, times_curves, sigs_chunk, dict_scatter)




class TraceViewer(BaseMultiChannelViewer):
    _default_params = default_params
    _default_by_channel_params = default_by_channel_params
    
    _ControllerClass = TraceViewer_ParamController
    
    request_data = QT.pyqtSignal(float, float, float, object, object, object, object)
    
    def __init__(self, **kargs):
        BaseMultiChannelViewer.__init__(self, **kargs)

        self.make_params()
        self.set_layout()
        self.make_param_controller()
        
        self.viewBox.doubleclicked.connect(self.show_params_controller)
        
        self.initialize_plot()
        
        self._xratio = 0.3
        
        self.last_sigs_chunk = None

        self.thread = QT.QThread(parent=self)
        self.datagrabber = DataGrabber(source=self.source, viewer=self)
        self.datagrabber.moveToThread(self.thread)
        self.thread.start()
        
        
        self.datagrabber.data_ready.connect(self.on_data_ready)
        self.request_data.connect(self.datagrabber.on_request_data)
        
        self.params.param('xsize').setLimits((0, np.inf))
        
    
    @classmethod
    def from_numpy(cls, sigs, sample_rate, t_start, name, channel_names=None,
                scatter_indexes=None, scatter_channels=None, scatter_colors=None):
        
        if scatter_indexes is None:
            source = InMemoryAnalogSignalSource(sigs, sample_rate, t_start, channel_names=channel_names)
        else:
            source = AnalogSignalSourceWithScatter(sigs, sample_rate, t_start, channel_names=channel_names,
                                            scatter_indexes=scatter_indexes, scatter_channels=scatter_channels, scatter_colors=scatter_colors)
        view = cls(source=source, name=name)
        
        
        
        
        return view

    def closeEvent(self, event):
        event.accept()
        self.thread.quit()
        self.thread.wait()
    
    def initialize_plot(self):
        
        self.vline = pg.InfiniteLine(angle = 90, movable = False, pen = '#00FF0055')
        self.plot.addItem(self.vline)
        
        self.curves = []
        self.channel_labels = []
        self.channel_offsets_line = []
        for c in range(self.source.nb_channel):
            color = self.by_channel_params['ch{}'.format(c), 'color']
            curve = pg.PlotCurveItem(pen='#7FFF00', downsampleMethod='peak', downsample=1,
                            autoDownsample=False, clipToView=True, antialias=False)#, connect='finite')
            self.plot.addItem(curve)
            self.curves.append(curve)
            
            ch_name = '{}: {}'.format(c, self.source.get_channel_name(chan=c))
            label = pg.TextItem(ch_name, color=color, anchor=(0, 0.5), border=None, fill=pg.mkColor((128,128,128, 180)))
            
            self.plot.addItem(label)
            self.channel_labels.append(label)

            offset_line = pg.InfiniteLine(angle = 0, movable = False, pen = '#7FFF00')
            self.plot.addItem(offset_line)
            self.channel_offsets_line.append(offset_line)
        
        if self.source.with_scatter:
            self.scatter = pg.ScatterPlotItem(size=10, pxMode = True)
            self.plot.addItem(self.scatter)
                
        
        
        self.viewBox.xsize_zoom.connect(self.params_controller.apply_xsize_zoom)
        self.viewBox.ygain_zoom.connect(self.params_controller.apply_ygain_zoom)
    
    def on_param_change(self, params=None, changes=None):
        #~ print('on_param_change')
        #track if new scale mode
        for param, change, data in changes:
            if change != 'value': continue
            if param.name()=='scale_mode':
                self.params_controller.compute_rescale()
            if param.name()=='antialias':
                for curve in self.curves:
                    curve.updateData(antialias=self.params['antialias'])
            
        
        self.refresh()
    
    def auto_scale(self):
        #~ print('auto_scale', self.last_sigs_chunk)
        if self.last_sigs_chunk is None:
            xsize = self.params['xsize']
            t_start, t_stop = self.t-xsize*self._xratio , self.t+xsize*(1-self._xratio)
            visibles, = np.nonzero(self.params_controller.visible_channels)
            gains = self.params_controller.gains
            offsets = self.params_controller.offsets
            _, _, _, _, _, _,sigs_chunk, _ = self.datagrabber.get_data(self.t, t_start, t_stop, gains, 
                                            offsets, visibles, self.params['decimation_method'])
            self.last_sigs_chunk = sigs_chunk
        
        self.params_controller.compute_rescale()
        self.refresh()
    
    def refresh(self):
        #~ print('TraceViewer.refresh', 't', self.t)
        xsize = self.params['xsize']
        t_start, t_stop = self.t-xsize*self._xratio , self.t+xsize*(1-self._xratio)
        visibles, = np.nonzero(self.params_controller.visible_channels)
        gains = self.params_controller.gains
        offsets = self.params_controller.offsets

        self.request_data.emit(self.t, t_start, t_stop, gains, offsets, visibles, self.params['decimation_method'])
        
    
    def on_data_ready(self, t,   t_start, t_stop, visibles, dict_curves, times_curves, sigs_chunk, dict_scatter):
        #~ print('on_data_ready', t, t_start, t_stop)
        
        if self.t != t:
            #~ print('on_data_ready not same t')
            return
        
        self.last_sigs_chunk = sigs_chunk
        offsets = self.params_controller.offsets
        self.graphicsview.setBackground(self.params['background_color'])
        
        for i, c in enumerate(visibles):
            self.curves[c].show()
            self.curves[c].setData(times_curves, dict_curves[c])
            
            color = self.by_channel_params['ch{}'.format(c), 'color']
            self.curves[c].setPen(color)
            
            if self.params['display_labels']:
                self.channel_labels[c].show()
                self.channel_labels[c].setPos(t_start, offsets[c])
                self.channel_labels[c].setColor(color)
            else:
                self.channel_labels[c].hide()
            
            if self.params['display_offset']:
                self.channel_offsets_line[c].show()
                self.channel_offsets_line[c].setPos(offsets[c])
                self.channel_offsets_line[c].setPen(color)
            else:
                self.channel_offsets_line[c].hide()
            
        for c in range(self.source.nb_channel):
            if c not in visibles:
                self.curves[c].hide()
                self.channel_labels[c].hide()
                self.channel_offsets_line[c].hide()
        
        if dict_scatter is not None:
            self.scatter.clear()
            all_x = []
            all_y = []
            all_brush = []
            for k, (x, y) in dict_scatter.items():
                all_x.append(x)
                all_y.append(y)
                color = self.source.scatter_colors.get(k, '#FFFFFF')
                all_brush.append(np.array([pg.mkBrush(color)]*len(x)))
            
            if len(all_x):
                all_x = np.concatenate(all_x)
                all_y = np.concatenate(all_y)
                all_brush = np.concatenate(all_brush)
                self.scatter.setData(x=all_x, y=all_y, brush=all_brush)

        self.vline.setPos(self.t)
        self.plot.setXRange( t_start, t_stop, padding = 0.0)
        self.plot.setYRange(self.params['ylim_min'], self.params['ylim_max'], padding = 0.0)
        
        #~ self.graphicsview.repaint()




import ephyviewer


from  ephyviewer.tests.testing_tools import make_fake_spiketrain_source


def test_spiketrain_viewer():
    source = make_fake_spiketrain_source()
    
    
    app = ephyviewer.mkQApp()
    view = ephyviewer.SpikeTrainViewer(source=source, name='spikes')
    
    win = ephyviewer.MainViewer(debug=True)
    win.add_view(view)
    win.show()
    
    app.exec_()
    
    
if __name__=='__main__':
    test_spiketrain_viewer()



import numpy as np
import ephyviewer
from  ephyviewer.tests.testing_tools import make_fake_signals, make_fake_signals_with_scatter


def test_traceviewer():
    source = make_fake_signals()

    
    app = ephyviewer.mkQApp()
    view = ephyviewer.TraceViewer(source=source, name='trace')
    #~ view.refresh()
    
    
    win = ephyviewer.MainViewer(debug=True, show_auto_scale=True)
    win.add_view(view)
    win.show()
    
    app.exec_()


def test_traceviewer_with_scatter():
    source = make_fake_signals_with_scatter()
    
    #~ exit()
    
    app = ephyviewer.mkQApp()
    view = ephyviewer.TraceViewer(source=source, name='trace_with_scatter')
    #~ view.refresh()
    
    
    win = ephyviewer.MainViewer(debug=True, show_auto_scale=True)
    win.add_view(view)
    win.show()
    
    app.exec_()    
    

if __name__=='__main__':
    test_traceviewer()
    #~ test_traceviewer_with_scatter()



# -*- coding: utf-8 -*-
#~ from __future__ import (unicode_literals, print_function, division, absolute_import)

from collections import OrderedDict

from .myqt import QT, QT_MODE
import pyqtgraph as pg


from .mainviewer import MainViewer
from .traceviewer import TraceViewer
from .epochviewer import EpochViewer
from .eventlist import EventList
from .spiketrainviewer import SpikeTrainViewer
from.tools import get_dict_from_group_param


from .datasource import get_source_from_neo

# browse neo.rawios and add some gui_params
from neo.rawio import rawiolist
rawio_gui_params = {}
rawio_gui_params['RawBinarySignal'] = [
        {'name': 'dtype', 'type': 'list', 'values':['int16', 'uint16', 'float32', 'float64']},
        {'name': 'nb_channel', 'type': 'int', 'value':1},
        {'name': 'sampling_rate', 'type': 'float', 'value':10000., 'step': 1000., 'suffix': 'Hz', 'siPrefix': True},
        {'name': 'bytesoffset', 'type': 'int', 'value':0},
    ]

all_neo_rawio_dict = OrderedDict()
for rawio_class in rawiolist:
    name = rawio_class.__name__.replace('RawIO', '')
    #~ print(name)
    rawio_class.name = name
    rawio_class.gui_params = rawio_gui_params.get(name, None)
    assert name not in all_neo_rawio_dict
    all_neo_rawio_dict[name] = rawio_class




class StandAloneViewer(MainViewer):
    def __init__(self):
        
        navigation_params = {}
        MainViewer.__init__(self, debug=False, settings_name=None, **navigation_params)
        
        self.navigation_dock.hide()
        
        self.create_actions_and_menu()
    
    
    def create_actions_and_menu(self):
        self.file_menu = self.menuBar().addMenu(self.tr("File"))
        
        do_open = QT.QAction('&Open', self, shortcut = "Ctrl+O")
        do_open.triggered.connect(self.open_dialog)
        self.file_menu.addAction(do_open)

    def open_dialog(self):
        dia = RawNeoOpenDialog()
        if dia.exec_():
            self.load_dataset(**dia.final_params)

    def load_dataset(self, neo_rawio_class=None, file_or_dir_names=[], io_params={}):
        #~ assert len(file_or_dir_names)==1
        if len(file_or_dir_names)!=1:
            return
        
        kargs = dict()
        if neo_rawio_class.rawmode.endswith('-file'):
            kargs['filename'] = file_or_dir_names[0]
        elif neo_rawio_class.rawmode.endswith('-dir'):        
            kargs['dirname'] = file_or_dir_names[0]
        kargs.update(io_params)
        
        self.neorawio = neo_rawio_class(**kargs)
        self.neorawio.parse_header()
        print(self.neorawio)
        
        self.navigation_dock.show()
        
        #TODO clear all
        sources = get_source_from_neo(self.neorawio)
        for i, sig_source in enumerate(sources['signal']):
            view = TraceViewer(source=sig_source, name='signal {}'.format(i))
            view.params['scale_mode'] = 'same_for_all'
            view.params['display_labels'] = True
            view.auto_scale()
            if i==0:
                self.add_view(view)
            else:
                self.add_view(view, tabify_with='signal {}'.format(i-1))
            

        for i, spike_source in enumerate(sources['spike']):
            view = SpikeTrainViewer(source=spike_source, name='spikes')
            self.add_view(view)

        for i, ep_source in enumerate(sources['epoch']):
            view = EpochViewer(source=ep_source, name='epochs')
            self.add_view(view)

            view = EventList(source=ep_source, name='Event list')
            self.add_view(view, location='bottom',  orientation='horizontal')




class RawNeoOpenDialog(QT.QDialog):
    def __init__(self, parent=None):
        QT.QDialog.__init__(self, parent = parent)
        
        self.setWindowTitle('Open file or dir with neo.rawio')
        self.setModal(True)
        
        self.resize(600, 300)

        layout = QT.QVBoxLayout()
        self.setLayout(layout)
        

        #for step 'source_type'
        self.tree_params = pg.parametertree.ParameterTree(parent=self)
        self.tree_params.header().hide()
        layout.addWidget(self.tree_params)
        self.tree_params.hide()
        
        # for filenames
        self.but_addfiles = QT.QPushButton('Select file')
        layout.addWidget(self.but_addfiles)
        self.but_addfiles.clicked.connect(self.on_addfiles)
        self.but_addfiles.hide()
        self.list_files = QT.QListWidget()
        layout.addWidget(self.list_files)
        self.list_files.hide() 
        layout.addStretch()
        
        self.but_next = QT.QPushButton('Next >')
        layout.addWidget(self.but_next)
        self.but_next.clicked.connect(self.validate_step)
        
        self.steps = ['source_type', 'filenames', 'io_params', ]
        self.display_step(self.steps[0])
        
        self.final_params = {}

    def display_step(self, step):
        self.actual_step = step
        
        if step=='source_type':
            source_types = list(all_neo_rawio_dict.keys())
            params = [{'name': 'format', 'type': 'list', 'values':source_types},
                            ]
            self.step_params = pg.parametertree.Parameter.create(name='Select format', type='group', children = params)
            self.tree_params.setParameters(self.step_params, showTop=True)
            self.tree_params.show()
        
        elif step=='filenames':
            self.but_addfiles.show()
            self.list_files.show()
        
        elif step=='io_params':
            params = self.neo_rawio_class.gui_params
            self.step_params = pg.parametertree.Parameter.create(name='Set params', type='group', children=params)
            self.tree_params.setParameters(self.step_params, showTop=True)
            self.tree_params.show()

  
    def validate_step(self):
        #~ print('validate_step', self.actual_step)
        step = self.actual_step
        if step=='source_type':
            self.neo_rawio_class = all_neo_rawio_dict[self.step_params['format']]
            self.final_params['neo_rawio_class'] = self.neo_rawio_class
            self.tree_params.hide()
            self.display_step('filenames')
            
        elif step=='filenames':
            file_or_dir_names = [self.list_files.item(i).text() for i in range(self.list_files.count())]
            if len(file_or_dir_names)==0: return
            self.final_params['file_or_dir_names'] = file_or_dir_names
            self.but_addfiles.hide()
            self.list_files.hide()
            
            if self.neo_rawio_class.gui_params is None:
                self.final_params['io_params'] = {}
                self.accept()

            else:
                self.display_step('io_params')
        
        elif step=='io_params':
            self.final_params['io_params'] = get_dict_from_group_param(self.step_params)
            self.tree_params.hide()
            self.display_step('mk_chan_grp')
            self.accept()
    
    
    def on_addfiles(self):
        #~ print('on_addfiles')
        #~ print(self.neo_rawio_class)
        #~ print(self.neo_rawio_class.rawmode)
        
        if self.neo_rawio_class.rawmode.endswith('-file'):
            fd = QT.QFileDialog(fileMode=QT.QFileDialog.ExistingFiles, acceptMode=QT.QFileDialog.AcceptOpen)
            #Todo play with rawio.extentsions
            fd.setNameFilters(['All (*)'])
        elif self.neo_rawio_class.rawmode.endswith('-dir'):
            fd = QT.QFileDialog(fileMode=QT.QFileDialog.DirectoryOnly, acceptMode=QT.QFileDialog.AcceptOpen)
            #~ fd.setNameFilters(['All (*)'])
        
        fd.setViewMode(QT.QFileDialog.Detail)
        if fd.exec_():
            filenames = fd.selectedFiles()
            self.list_files.addItems(filenames)







from setuptools import setup
import os


install_requires = [
                    'numpy',
                    #~ 'PyQt5',
                    'pyqtgraph>=0.10.0',
                    'matplotlib>=2.0',
                    'scipy',
                    ]

long_description = ""

#~ import ephyviewer
#~ version = ephyviewer.__version__,
with open('ephyviewer/version.py') as f:
    version = f.readline()[:-1].split('=')[1].replace(' ', '').replace("'", "")


entry_points={'console_scripts': ['ephyviewer=ephyviewer.scripts:launch_standalone_ephyviewer']}


setup(
    name = "ephyviewer",
    version=version,
    packages = ['ephyviewer', 'ephyviewer.datasource', 'ephyviewer.tests'],
    install_requires=install_requires,
    author = "S.Garcia",
    author_email = "sam.garcia.die@gmail.com",
    description = "Simple viewers for ephy stuff",
    entry_points = entry_points,
    long_description = long_description,
    license = "MIT",
    url='https://github.com/NeuralEnsemble/ephyviewer',
    classifiers = [
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        ]
)



import ephyviewer



def test_base():
    app = ephyviewer.mkQApp()
    toolbar = ephyviewer.NavigationToolBar()
    toolbar.show()
    app.exec_()
    
    
    
if __name__=='__main__':
    test_base()

